@@ -322,7 +322,7 @@ i915_emit_const1f(struct i915_fragment_program * p, GLfloat c0)
       }
    }
 
-   fprintf(stderr, "%s: out of constants\n", __FUNCTION__);
+   fprintf(stderr, "%s: out of constants\n", __func__);
    p->error = 1;
    return 0;
 }
@@ -359,7 +359,7 @@ i915_emit_const2f(struct i915_fragment_program * p, GLfloat c0, GLfloat c1)
       }
    }
 
-   fprintf(stderr, "%s: out of constants\n", __FUNCTION__);
+   fprintf(stderr, "%s: out of constants\n", __func__);
    p->error = 1;
    return 0;
 }
@@ -391,7 +391,7 @@ i915_emit_const4f(struct i915_fragment_program * p,
       }
    }
 
-   fprintf(stderr, "%s: out of constants\n", __FUNCTION__);
+   fprintf(stderr, "%s: out of constants\n", __func__);
    p->error = 1;
    return 0;
 }
@@ -430,7 +430,7 @@ i915_emit_param4fv(struct i915_fragment_program * p, const GLfloat * values)
       }
    }
 
-   fprintf(stderr, "%s: out of constants\n", __FUNCTION__);
+   fprintf(stderr, "%s: out of constants\n", __func__);
    p->error = 1;
    return 0;
 }
@@ -517,18 +517,18 @@ i915_fini_program(struct i915_fragment_program *p)
    }
 
    if (p->error) {
-      p->FragProg.Base.NumNativeInstructions = 0;
-      p->FragProg.Base.NumNativeAluInstructions = 0;
-      p->FragProg.Base.NumNativeTexInstructions = 0;
-      p->FragProg.Base.NumNativeTexIndirections = 0;
+      p->FragProg.arb.NumNativeInstructions = 0;
+      p->FragProg.arb.NumNativeAluInstructions = 0;
+      p->FragProg.arb.NumNativeTexInstructions = 0;
+      p->FragProg.arb.NumNativeTexIndirections = 0;
    }
    else {
-      p->FragProg.Base.NumNativeInstructions = (p->nr_alu_insn +
+      p->FragProg.arb.NumNativeInstructions = (p->nr_alu_insn +
                                                 p->nr_tex_insn +
                                                 p->nr_decl_insn);
-      p->FragProg.Base.NumNativeAluInstructions = p->nr_alu_insn;
-      p->FragProg.Base.NumNativeTexInstructions = p->nr_tex_insn;
-      p->FragProg.Base.NumNativeTexIndirections = p->nr_tex_indirect;
+      p->FragProg.arb.NumNativeAluInstructions = p->nr_alu_insn;
+      p->FragProg.arb.NumNativeTexInstructions = p->nr_tex_insn;
+      p->FragProg.arb.NumNativeTexIndirections = p->nr_tex_indirect;
    }
 
    p->declarations[0] |= program_size + decl_size - 2;
